//Copyright (c) 2017 by Michael Tracy
//Version 3.0 Added "Pages before/after"
//Version 2.2.2 Fixed issue with non-overlapping citations
//Version 2.2.1 Fixed issue with overlapping citations.
//Version 2.2
//Added Lines Per Page to work with transcripts with only 24 lines.
//Added presistence so it save the last used parameters.
//moved the menu to the Edit menu for compatibility.
//fixed issue with overlapping citations being duplicated
//Version 2.1
//Added "Box" annotation type and "Black" color.  This will be Gray if used to Highlight.
//This program is free to use, but you cannot modify, sell, or re-distribute any portion of it.
//There is no warranty of any aspect of this software.



function GetFirstWord(oDoc, nPageNum)
{

     var len = oDoc.getPageNumWords(nPageNum);
       // Loop over words on page
       for(var i=0;i<len;i++)
       {
          var wd = oDoc.getPageNthWord(nPageNum,i,false);
          wd = wd.replace(/^\s+|\s+$/g,"");
          if (wd == "1")
                {
                    return i;
                }
       }

    throw("Can't find the first line on page: " + nPageNum);
 
}


function GetLineEnd(oDoc, nPageNum, nLineStart, nLineNum, nLinesPerPage) {
    var nPosition = oDoc.getPageNthWordQuads(nPageNum, nLineStart, false)[0][7];

    var len = oDoc.getPageNumWords(nPageNum);

    for(var i=nLineStart;i<len;i++)
      {
              var nCurPos = oDoc.getPageNthWordQuads(nPageNum, i , false)[0][7];
              if ((nCurPos > nPosition + 6) || (nCurPos < nPosition - 6))
              {
                return i-1;
              }
       }
    if ( nLineNum == nLinesPerPage )
    {
        return i-1;
    }    
    throw("Can't find the end of line: " + nLineNum + " on page: " + nPageNum);   
}


function GetLineStart(oDoc, nPageNum, nLineNumber, nFirstWord, nStartAt)
{
     var qdFirstWord =oDoc.getPageNthWordQuads(nPageNum, nFirstWord, false);
    var nPosition = qdFirstWord[0][6];

    var len = oDoc.getPageNumWords(nPageNum);

    for(var i=nStartAt;i<len;i++)
       {
          var wd = oDoc.getPageNthWord(nPageNum,i,false);
              if (wd == nLineNumber )
                {
                    var nCurPos = oDoc.getPageNthWordQuads(nPageNum, i, false)[0][6];  
                    if (nCurPos  + 12 > nPosition &&  nCurPos  - 12 < nPosition )
                   {
                        return i;
                   }
                }
       }
        
    throw("Can't find line number: " + nLineNumber + "on page: " + 
nPageNum);   
}


function HighlightCitation(oDoc, oCitation, oDlg)
{
	console.println("Citation:" + oCitation.nStartPage + ":" + oCitation.nStartLine + "-" + oCitation.nEndPage + ":" + oCitation.nEndLine);
    
    if(oCitation.nStartLine == -1)
	{
			// a unmarked page is passed in as a -1, it is used for PagesBefore/After. 
			return;
	}
    if (oCitation.nStartPage == oCitation.nEndPage)
    {
        HighlightLines(oDoc, oCitation.nStartPage, oCitation.nStartLine, oCitation.nEndLine, false, oDlg);
        return;
    }

    var nCurrentStartLine = oCitation.nStartLine;
    for ( var nCurrentPage = oCitation.nStartPage; nCurrentPage <= oCitation.nEndPage; nCurrentPage++)
    {
        var nCurrentEndLine = oDlg.nLinesPerPage;
        if (nCurrentPage == oCitation.nEndPage)
        {
            HighlightLines(oDoc, nCurrentPage, 1, oCitation.nEndLine, false, oDlg);
        }
        else
        {
            HighlightLines(oDoc, nCurrentPage, nCurrentStartLine, nCurrentEndLine, true, oDlg);
        }
        nCurrentStartLine = 1;
    }
}

function BracketLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, offset, strColor)
{
	nPage = nPage - 1;
	var nFirstWord = GetFirstWord(oDoc, nPage);
	var nCurrentWord = nFirstWord ;
	
	var nStartLine = GetLineStart(oDoc, nPage, nStartLine, nFirstWord, nCurrentWord);
	var nEndLine = GetLineStart(oDoc, nPage, nEndLine, nFirstWord, nCurrentWord);
	
	var qdStart = oDoc.getPageNthWordQuads(nPage,nStartLine);
	var qdEnd = oDoc.getPageNthWordQuads(nPage,nEndLine);
	
	var col = color.black
		
	if (strColor == "Green") { col = color.green;}
	if (strColor == "Blue") { col = color.blue ;}
	if (strColor == "Pink") {col = color.red ;}
	if (strColor == "Yellow") {col = color.yellow ;}
	
	strArrowEnd = "Butt";
	if (bIsMultiPage) { strArrowEnd = "OpenArrow" }
	oDoc.addAnnot({page: nPage, strokeColor: col, type: "Line", width: 3, 
		arrowBegin: "Butt", arrowEnd: strArrowEnd, 
		points: WordQuadsToPoints(qdStart, qdEnd, offset), });
}
	

function BoxLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, strColor, offset, rightOffset)
{

	nPage = nPage - 1;
	var nFirstWord = GetFirstWord(oDoc, nPage);
	var nCurrentWord = nFirstWord ;
	
	var nStartLine = GetLineStart(oDoc, nPage, nStartLine, nFirstWord, nCurrentWord);
	var nEndLine = GetLineStart(oDoc, nPage, nEndLine, nFirstWord, nCurrentWord);
	
	var qdStart = oDoc.getPageNthWordQuads(nPage,nStartLine);
	var qdEnd = oDoc.getPageNthWordQuads(nPage,nEndLine);
	
	var col = color.black
	
	if (strColor == "Green") { col = color.green;}
	if (strColor == "Blue") { col = color.blue ;}
	if (strColor == "Pink") {col = color.red ;}
	if (strColor == "Yellow") {col = color.yellow ;}	
		
	if (bIsMultiPage)
	{
		oDoc.addAnnot({page: nPage, strokeColor: col, type: "PolyLine", width: 2, 
			arrowBegin: "OpenArrow", arrowEnd: "OpenArrow" ,   
			vertices: WordQuadsToBox(qdStart, qdEnd, offset, rightOffset), });
	}
	else
	{
	oDoc.addAnnot({page: nPage, strokeColor: col, type: "Polygon", width: 2,  
		vertices: WordQuadsToBox(qdStart, qdEnd, offset, rightOffset), });
	}
}

function WordQuadsToBox(qdStart, qdEnd, offset, rightOffset)
{
	return [ 
		[(qdEnd[0][4])-offset,(qdEnd[0][5]-7)] , 
		[(qdEnd[0][4])-offset,(qdStart[0][1]+7)], 
		[(612-rightOffset), (qdStart[0][1]+7)],
		[(612-rightOffset), (qdEnd[0][5]-7)]
		];

}


function HighlightLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, oDlg)
{
	var strAnnotType = oDlg.strAnnotType;
	var strColor = oDlg.strColor;
	var offset = oDlg.nOffset;
	var rightOffset = oDlg.nRightOffset;
	var nLinesPerPage = oDlg.nLinesPerPage;
	

	if (strAnnotType == "Bracket")
	{
		BracketLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, offset, strColor);
		return;
	}
	
	if (strAnnotType == "Box")
	{
		BoxLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, strColor, offset, rightOffset);
		return;
	}

	
	if (strAnnotType == "Both")
	{
		BracketLines(oDoc, nPage, nStartLine, nEndLine, bIsMultiPage, offset, "Black");
		strAnnotType = "Highlight";
	}
	
    nPage = nPage - 1;
    var nFirstWord = GetFirstWord(oDoc, nPage);
    var nCurrentWord = nFirstWord ;

    for (var nCurLine = nStartLine; nCurLine<=nEndLine; nCurLine++)
    {

        var nStartWord = GetLineStart(oDoc, nPage, nCurLine, nFirstWord, nCurrentWord);
        var nEndWord = GetLineEnd(oDoc, nPage, nStartWord, nCurLine, nLinesPerPage);
        nCurrentWord = nEndWord;        

        var qdStart = oDoc.getPageNthWordQuads(nPage,nStartWord);
        var qdEnd = oDoc.getPageNthWordQuads(nPage,nEndWord);
		
		var col = color.yellow;
		var opac = .6 // yellow doesn't print as well, so we use a default higher opacity
		
		if (strColor == "Green") { col = color.green; opac = .4; }
		if (strColor == "Blue") { col = color.blue ; opac = .4;}
		if (strColor == "Pink") {col = color.red ; opac = .4;}
		if (strColor == "Black") {col = color.black ; opac = .2;}
		
		if (strAnnotType == "Underline") { strAnnotType = "Underline"; opac = 1 } // underlining will be too light without this.
		
		oDoc.addAnnot({page: nPage, strokeColor: col, type: strAnnotType, opacity: opac,
		quads: MergeQuads(qdStart, qdEnd, offset), });
    }
    return 1;
}

function AddUnmarkedPages(nStartPage, nEndPage)
{
	var oExtraPages = new Object();
	oExtraPages.nStartPage = nStartPage;
	oExtraPages.nEndPage = nEndPage;
	oExtraPages.nStartLine = -1;
	return oExtraPages;
}

function InsertUnmarkedPages(oCitation, oPrevious, arOutput, nPagesBefore, nPagesAfter, nFirstPage)
{
        // Insert 'blank' citations for PagesBefore and After       
        if (nPagesBefore > 0)
        {
        	// add 'blank' entries for Pages Before
        	if (oPrevious == undefined)
        	{
        		var nStartUnmarkedPage;
        		
        		nStartUnmarkedPage = oCitation.nStartPage - nPagesBefore;
        		if (nStartUnmarkedPage < nFirstPage)
        		{
        			nStartUnmarkedPage = nFirstPage;
        		}
        		//TODO: check other boudaries for header
        		
        		arOutput.push(AddUnmarkedPages(nStartUnmarkedPage, oCitation.nStartPage - 1));
        	}
        	else
        	{
        		// this is not the first citation
        		if(oCitation.nStartPage - nPagesBefore <= oPrevious.nEndPage + nPagesAfter )
        		{
        			// included pages overlap with a prior citation
        			if (oCitation.nStartPage - 1 <= oPrevious.nEndPage)
        			{
        			//do nothing
        			}
        			else
        			{
        				// add all the pages in between
        				arOutput.push(AddUnmarkedPages(oPrevious.nEndPage + 1, oCitation.nStartPage -1 ));
        			}
        		}
        		else
        		{
        			// no overlap between the "before" and "after"
        			if (nPagesAfter > 0)
        			{
        				arOutput.push(AddUnmarkedPages(oPrevious.nEndPage + 1, oPrevious.nEndPage + nPagesAfter));
        					
        			}

					arOutput.push(AddUnmarkedPages(oCitation.nStartPage - nPagesBefore, oCitation.nStartPage -1));

        		}
        	}
        }

}

function MergeCitations(arCitations, nFirstPage, nPagesBefore, nPagesAfter)
{
    var arOutput = new Array();
    var oPrevious;

    for (var nCurrent=0; nCurrent< arCitations.length; nCurrent++)
    {
        if (arCitations[nCurrent].length < 3)
        {
            continue;
        }
        console.println("Old:" + arCitations[nCurrent]);
        var oCitation = SplitCitation(arCitations[nCurrent]);

        InsertUnmarkedPages(oCitation, oPrevious, arOutput, nPagesBefore, nPagesAfter, nFirstPage);


        
        if (oPrevious != undefined )
        {
        	// skip all completely overlaping
        	if (oPrevious.nStartPage == oCitation.nStartPage && oPrevious.nStartLine == oCitation.nStartLine
        		&& oPrevious.nEndPage == oCitation.nEndPage && oPrevious.nEndLine == oCitation.nEndLine )
        	{
        			continue;
			}
			// skip all with completley overlapping pages
			if ( oPrevious.nEndPage > oCitation.nEndPage )
			{
				continue;
             }
             
             //skip all with overlapping end citations. 
             if ( oPrevious.nEndPage == oCitation.nEndPage && oPrevious.nEndLine >= oCitation.nEndLine)
             {
             	continue;
             }
             
             // merge those with overlapping pages
            if (oPrevious.nEndPage > oCitation.nStartPage || ( oPrevious.nEndPage == oCitation.nStartPage && oPrevious.nEndLine >= oCitation.nStartLine) )
            {
				
                oPrevious.nEndPage = oCitation.nEndPage;
                oPrevious.nEndLine = oCitation.nEndLine;
                arOutput.pop();
                arOutput.push(oPrevious);
                continue;
            }
         }
        arOutput.push(oCitation);
        oPrevious = oCitation;                
    }
    if (nPagesAfter > 0)
    {
    	// add unmarked pages to last citation
    	arOutput.push(AddUnmarkedPages(oCitation.nEndPage + 1, oCitation.nEndPage + nPagesAfter));
    	//TODO handle if this pushes it past the end of the file, not critical because generally there are index, signature, etc. 
    }
    return arOutput;
}

function SplitCitation(strCitation)
{
    var oCitation = new Object();
    var strFirst = strCitation.split("-")[0];
    var strSecond = strCitation.split("-")[1];

    if (strSecond == undefined)
    {
        throw("Citation: " + strCitation + " is not in the proper format. Should be Page:line-line, example 10:1-8 or 10:1-11:8 if more than one page.");
    }
    
    oCitation.nStartPage = strFirst.split(":")[0] * 1;
    oCitation.nStartLine = strFirst.split(":")[1] * 1;

    oCitation.nEndPage = oCitation.nStartPage;
    oCitation.nEndLine = strSecond * 1;

    if (strSecond.split(":")[1] != undefined)
    {
        oCitation.nEndPage = strSecond.split(":")[0] * 1;
        oCitation.nEndLine = strSecond.split(":")[1] * 1;
    }

    if (oCitation.nStartPage == NaN || oCitation.nStartLine == NaN ||
        oCitation.nEndPage == NaN || oCitation.nEndLine == NaN )
    {
        throw("Citation: " + strCitation + " is not in the proper format. Should be Page:line-line, example 10:1-8 or 10:1-11:8 if more than one page.");
    }
    if (oCitation.nStartLine > global.nLinesPerPage || oCitation.nEndLine > global.nLinesPerPage  )
    {
        throw("Citation: " + strCitation + " is not in the proper format. Line numbers can not be greater than " + global.nLinesPerPage.toString() );
    }
    return oCitation;
}


function MergeQuads(quad1, quad2)
{
    return [[quad1[0][0],quad1[0][1],quad2[0][2],quad2[0][3],
        quad1[0][4],quad1[0][5],quad2[0][6],quad2[0][7]]];
}

function WordQuadsToPoints(quad1, quad2, offset)
{
	return [ [(quad2[0][4])-offset,quad1[0][1]],[(quad2[0][4])-offset,quad2[0][5]] ];
}

function ComboIsSelected(nValSelected, nListVal)
{
	if (nValSelected == nListVal)
	{
		return nListVal;
	}
	return -1 * nListVal;
	
}


function OpenDialog()
{
// Dialog Definition
var oDlg =
{
   strStartPage: "1",
   strCitations: "",
   strHeader: "",
   strSignature: "",
   nPagesBefore: "0",
   nPagesAfter: "0",
   bExtractPages: true,
   strAnnotType: "Highlight",
   strColor: "Yellow",
   nOffset: 18,
   nRightOffset: 18,
   nLinesPerPage: 25,
   
    initialize: function(dialog)
   {

      dialog.load({"strt":this.strStartPage});
      dialog.load({"pagb":this.nPagesBefore});
      dialog.load({"paga":this.nPagesAfter});
      if (this.bExtractPages == true) {	dialog.load({"rd01": true });}  else {	dialog.load({"rd02": true });}
      
      if (this.strColor == "Yellow") {dialog.load({"cl01": true }); }
      if (this.strColor == "Green") {dialog.load({"cl02": true }); }
      if (this.strColor == "Blue") {dialog.load({"cl03": true }); }
      if (this.strColor == "Pink") {dialog.load({"cl04": true }); }
      if (this.strColor == "Black") {dialog.load({"cl05": true }); }
      
      
      if (this.strAnnotType == "Highlight") { dialog.load({"hl01": true });}
      if (this.strAnnotType == "Bracket") { dialog.load({"hl02": true });}
      if (this.strAnnotType == "Both") { dialog.load({"hl03": true });}
      if (this.strAnnotType == "Underline") { dialog.load({"hl04": true });}
      if (this.strAnnotType == "Box") { dialog.load({"hl05": true });}
	
      
      dialog.load({"ofst": {"0.25 inches":ComboIsSelected( this.nOffset, 18), "0.5 inches":ComboIsSelected( this.nOffset, 36), 
    	"0.75 inches":ComboIsSelected( this.nOffset, 54), "1 inch":-72,"1.25 inches":ComboIsSelected( this.nOffset, 90)}});
      dialog.load({"rfst": {"0.25 inches":ComboIsSelected( this.nOffset, 18), "0.5 inches":ComboIsSelected( this.nOffset, 36), 
      	"0.75 inches":ComboIsSelected( this.nOffset, 54), "1 inch":-72,"1.25 inches":ComboIsSelected( this.nOffset, 90)}});
      	
      dialog.load({"lnpp": 
      	{
      	"23 Lines":ComboIsSelected( this.nLinesPerPage, 23),
      	"24 Lines":ComboIsSelected( this.nLinesPerPage, 24), 
      	"25 Lines":ComboIsSelected( this.nLinesPerPage, 25), 
      	"26 Lines":ComboIsSelected( this.nLinesPerPage, 26),
      	"27 Lines":ComboIsSelected( this.nLinesPerPage, 27),
      	"28 Lines":ComboIsSelected( this.nLinesPerPage, 28),
      	"29 Lines":ComboIsSelected( this.nLinesPerPage, 29),
      	"30 Lines":ComboIsSelected( this.nLinesPerPage, 30)
      	}});
   },
   commit: function(dialog)
   {
    var data = dialog.store();
    this.strStartPage = data[ "strt"];
    this.strCitations = data[ "cite"];
    this.strHeader = data["hedr"];
    this.strSignature = data["sign"];
    this.bExtractPages = data["rd01"];
    this.nPagesBefore = data["pagb"];
    this.nPagesAfter = data["paga"];
    
    if (data["cl01"]) { this.strColor = "Yellow"; }
    if (data["cl02"]) { this.strColor = "Green"; }
    if (data["cl03"]) { this.strColor = "Blue"; }
    if (data["cl04"]) { this.strColor = "Pink"; }
    if (data["cl05"]) { this.strColor = "Black"; }
    
    var elements = dialog.store()["ofst"]
    for(var i in elements) 
    {
    	if (elements[i] > 0) 
    	{
    		this.nOffset = elements[i];
    	}   	
    }
    
	var relements = dialog.store()["rfst"]
	for(var i in relements) 
	{
		if (relements[i] > 0) 
		{
			this.nRightOffset = relements[i];
		}   	
	}
    
	var lineselements = dialog.store()["lnpp"]
	for(var i in lineselements) 
	{
		if (lineselements[i] > 0) 
		{
			this.nLinesPerPage = lineselements[i];
		}   	
	}
    
    this.strAnnotType = "Highlight";
    
    if (data["hl02"]) 
    {
    	this.strAnnotType = "Bracket";
    }
	if (data["hl03"]) 
	{
		this.strAnnotType = "Both";
	}
	if (data["hl04"]) 
	{
		this.strAnnotType = "Underline";
	}
	if (data["hl05"]) 
	{
		this.strAnnotType = "Box";
	}
    
   },
   description:
   {
      name: "Transcript Marker",
      elements:
      [
       {
        type: "view",
        elements:
        [
        {
		         
		         type: "view",		 
				 				align_children: "align_row",		 
				 				elements:
				 
		[
        
			{
			 name: "Start Page:",
			 type: "static_text",
			 },
			 {
			 item_id: "strt",
			 type: "edit_text",
			 char_width: 5,
			 },
		]
		},
        {
         
         type: "view",		 
		 align_children: "align_row",		 
		 elements:
		 [
			 {
			  name: "Header:    ",
			  type: "static_text",
			  },
			  {
			  item_id: "hedr",
			  type: "edit_text",
			  char_width: 10,
			 },
			 {
			   name: "Signature:",
			   type: "static_text",
			   },
			   {
			   item_id: "sign",
			   type: "edit_text",
			   char_width: 10,
			 }
         ]
		},
         
         {
		 		         
		 		         type: "view",		 
		 				 				align_children: "align_row",		 
		 				 				elements:
		 				 
		 		[
		 			 {
		 			   name: "Lines Per Page:",
		 			   type: "static_text",
		 			   },
		 			   {
		 			   item_id: "lnpp",
		 			   type: "popup",
		 			   width: 50,
		 			 }
		          ]
         },
         
         
         {
		 
			type: "view",		 
			align_children: "align_row",		 
			elements:

			[		 
				{		 
					type: "static_text",
					name: "Extract Pages:"

				},

				{

					type: "radio",
					item_id: "rd01",
					group_id: "rado",
					name: "Yes"

				},

				{

					type: "radio",
					item_id: "rd02",		 
					group_id: "rado",		 
					name: "No",

				}		 
			]		 
		},
		{

				type: "view",		 
				align_children: "align_row",		 
				elements:

				[		 
					{		 
						type: "static_text",
						name: "Color: "

					},

					{

						type: "radio",
						item_id: "cl01",
						group_id: "colo",
						name: "Yellow"

					},

					{

						type: "radio",
						item_id: "cl02",		 
						group_id: "colo",		 
						name: "Green",

					},		

					{

						type: "radio",
						item_id: "cl03",		 
						group_id: "colo",		 
						name: "Blue",

					},
					{

						type: "radio",
						item_id: "cl04",		 
						group_id: "colo",		 
						name: "Pink",

					},
					{

						type: "radio",
						item_id: "cl05",		 
						group_id: "colo",		 
						name: "Black",

					}
				]		 
		},
		{
				 
					type: "view",		 
					align_children: "align_row",		 
					elements:
		
					[		 
		
						{
		
							type: "radio",
							item_id: "hl01",
							group_id: "hilo",
							name: "Highlight"
		
						},
						
						{

							type: "radio",
							item_id: "hl02",
							group_id: "hilo",
							name: "Bracket"

						},

						{

							type: "radio",
							item_id: "hl03",
							group_id: "hilo",
							name: "Both"

						},
		
						{
		
							type: "radio",
							item_id: "hl04",		 
							group_id: "hilo",		 
							name: "Underline",
		
						},
						
						{

							type: "radio",
							item_id: "hl05",		 
							group_id: "hilo",		 
							name: "Box",

						}
					]		 
		},
		
		{
				 
					type: "view",		 
					align_children: "align_row",		 
					elements:
		
					[		 
						{		 
							type: "static_text",
							name: "Left Offset:"
		
						},
		
						{
		
							type: "popup",
							item_id: "ofst",
							width: 100
						},
						{		 
							type: "static_text",
							name: "Right Offset:"

						},

						{

							type: "popup",
							item_id: "rfst",
							width: 100
						}
					]		 
		},

        {
         
         type: "view",		 
		 align_children: "align_row",		 
		 elements:
		 [
			 {
			  name: "Pages Before:",
			  type: "static_text",
			  },
			  {
			  item_id: "pagb",
			  type: "edit_text",
			  char_width: 10,
			 },
			 {
			   name: "Pages After:",
			   type: "static_text",
			   },
			   {
			   item_id: "paga",
			   type: "edit_text",
			   char_width: 10,
			 }
         ]
		},
		
        {
         name: "Citations:",
         type: "static_text",
         },
         {
         item_id: "cite",
         type: "edit_text",
         char_width: 15,
         char_height: 30,
         multiline: true
         },
         {
         	type: "static_text",
         	name: "Copyright 2017 Law Offices of Michael Tracy. Free to use. See code for license."
         },
         {
         type: "ok_cancel",
         },
        ]
       },
     ]
   }
};

// Dialog Activation

SetupParameters(oDlg);

if ("ok" == app.execDialog(oDlg))
{
	StoreParameters(oDlg);
    return oDlg;
}
return undefined;
}

function StoreParameters(oDlg)
{
	global.strAnnotType = oDlg.strAnnotType;
	global.setPersistent("strAnnotType",true);
	
	global.bExtractPages = oDlg.bExtractPages;
	global.setPersistent("bExtractPages",true);
	
	global.strColor = oDlg.strColor;
	global.setPersistent("strColor",true);
	
	global.nOffset = oDlg.nOffset;
	global.setPersistent("nOffset",true);
	
	global.nRightOffset = oDlg.nRightOffset;
	global.setPersistent("nRightOffset",true);
	
	global.nLinesPerPage = oDlg.nLinesPerPage;
	global.setPersistent("nLinesPerPage", true);
	
	global.nPagesBefore = oDlg.nPagesBefore;
	global.setPersistent("nPagesBefore", true);
	
	global.nPagesAfter = oDlg.nPagesAfter;
	global.setPersistent("nPagesAfter", true);
	

}
	
function SetupParameters(oDlg)
{
	
	oDlg.strAnnotType = "Both";
	if (global.strAnnotType!=null) { oDlg.strAnnotType = global.strAnnotType ; }
	
	oDlg.bExtractPages = true;
	if (global.bExtractPages!=null) { oDlg.bExtractPages = global.bExtractPages ; }
	
	oDlg.strColor = "Yellow";
	if (global.strColor!=null) { oDlg.strColor = global.strColor ; }
	
	oDlg.nOffset = 18;
	if (global.nOffset!=null) { oDlg.nOffset = global.nOffset ; }
	
	oDlg.nRightOffset = 18;
	if (global.nRightOffset!=null) { oDlg.nRightOffset = global.nRightOffset ; }
	
	oDlg.nLinesPerPage = 25;
	if (global.nLinesPerPage != null) { oDlg.nLinesPerPage = global.nLinesPerPage ; }
	
	oDlg.nPagesBefore = 0;
	if (global.nPagesBefore != null) { oDlg.nPagesBefore = global.nPagesBefore ; }
	
	oDlg.nPagesAfter = 0;
	if (global.nPagesAfter != null) { oDlg.nPagesAfter = global.nPagesAfter ; }
	
}

trustedProcess = app.trustedFunction(

function ProcessSplit()
{

    try
    {
        var nTotalPages = this.numPages;
        var oDlg = OpenDialog();
        if (oDlg == undefined)
        {
            return;
        }
        var nFirstPage = oDlg.strStartPage * 1;
        var nPagesBefore = oDlg.nPagesBefore * 1;
        var nPagesAfter = oDlg.nPagesAfter * 1;
        
        if (nFirstPage < 1) { nFirstPage = 1; }
        var strCitations = oDlg.strCitations;
        var arCitations = strCitations.split("\r\n");
        arCitations.sort(sortByPage);
        var arMerged=MergeCitations(arCitations,nFirstPage,nPagesBefore,nPagesAfter);
		AdjustPages(arMerged, nFirstPage);

        //app.beginPriv();
        var re = /\.pdf$/i;
        var filename = this.path;

		if (oDlg.bExtractPages)
		{
			var oDoc = app.newDoc();
			var nPageCount = 0;
			var nPreviousEndPage = 0;

			for (var i = 0; i < arMerged.length; i++)
			{

				if ( ((arMerged[i].nEndPage - 1) <= nTotalPages) && (arMerged[i].nStartPage > 0)  )
				{
					if (i > 0 && nPreviousEndPage == arMerged[i].nStartPage)
					{
						// the pages overlap, but the actual citations do not.
						if (arMerged[i].nEndPage > arMerged[i].nStartPage)
						{
							oDoc.insertPages({nPage: oDoc.numPages - 1, nStart: arMerged[i].nStartPage, nEnd: arMerged[i].nEndPage - 1,
												cPath: filename, });
						}
						var nOldPageCount = nPageCount;
						nPageCount = nPageCount + arMerged[i].nEndPage - arMerged[i].nStartPage;
						arMerged[i].nStartPage = nOldPageCount;
						nPreviousEndPage = arMerged[i].nEndPage;
						arMerged[i].nEndPage = nPageCount;
					}
					else
					{
						oDoc.insertPages({nPage: oDoc.numPages - 1, nStart: arMerged[i].nStartPage - 1, nEnd: arMerged[i].nEndPage - 1,
								cPath: filename, });
						var nOldPageCount = nPageCount;
						nPageCount = nPageCount + arMerged[i].nEndPage - arMerged[i].nStartPage + 1;
						arMerged[i].nStartPage = nOldPageCount + 1;
						nPreviousEndPage = arMerged[i].nEndPage;
						arMerged[i].nEndPage = nPageCount;
					}
				}
				else
				{
					app.alert("Citation: " + (arMerged[i].nStartPage + nFirstPage - 1 ) + ":" + arMerged[i].nStartLine +
						"-" + (arMerged[i].nEndPage +nFirstPage - 1 )+ ":" + arMerged[i].nEndLine + " is not in this PDF file and will be skipped.");
					arMerged.splice(i,1);
					i--;
				}
			}


			if (nPageCount > 0)
			{
				oDoc.deletePages({nStart: 0, nSend:0,});
			}
        //app.endPriv()
		}
		else
		{
			oDoc=this;
		}
		
        for (var i = 0; i < arMerged.length; i++)
        {
           HighlightCitation(oDoc, arMerged[i], oDlg);
        }
		
		if (oDlg.bExtractPages)
		{
			if (oDlg.strHeader.length > 0)
			{
				InsertHeader(oDoc, oDlg.strHeader, filename, nFirstPage, nTotalPages);
			}

			if (oDlg.strSignature.length > 0)
			{
				InsertFooter(oDoc, oDlg.strSignature, filename, nFirstPage, nTotalPages);
			}

			RemoveLinks(oDoc);
		}
    }
    catch(error)
    {
        app.alert({cMsg: error});
    }

}
)

function InsertHeader(oDoc, strHeader, strPath, nFirstPage, nTotalPages)
{
    InsertPages(-1, oDoc, strHeader, strPath, nFirstPage, nTotalPages);
}

function InsertFooter(oDoc, strFooter, strPath, nFirstPage, nTotalPages)
{
	InsertPages(oDoc.numPages - 1 , oDoc, strFooter, strPath, nFirstPage, nTotalPages);
}

function InsertPages(nInsertAt, oDoc, strPages, strPath, nFirstPage, nTotalPages)
{
    if (strPages.length > 0)
    {
        var nStartPage = strPages.split("-")[0];
        var nEndPage = strPages.split("-")[1];
        if (nEndPage == undefined)
        {
            nEndPage = nStartPage;
        }
    
		if (nStartPage - 1 - nFirstPage + 1 >= nTotalPages ||  nEndPage - 1 - nFirstPage + 1 >= nTotalPages || nStartPage - 1 - nFirstPage + 1 < 0)
		{
			app.alert("Citation: " + strPages + " does not exist in this document. Wrong page numbers.");
		}
		else
		{
			oDoc.insertPages({nPage: nInsertAt, nStart: nStartPage - 1 - nFirstPage + 1, nEnd: nEndPage - 1 - nFirstPage + 1,
				cPath: strPath, });
		}
	}
}

function sortByPage(a, b)
{
    var x = a.split(":")[0] * 1;
    var y = b.split(":")[0] * 1;

    var line1 = a.split("-")[0].split(":")[1] * 1;
    var line2 = b.split("-")[0].split(":")[1] * 1;

    
    if (x < y) return -1;
    if (x > y) return 1;
    if (line1 < line2) return -1;
    if (line1 > line2) return 1;
    return 0;
}

function AdjustPages(arMerged, nStartPage)
{
	if (nStartPage <= 1)
		return;
		
    for (var i = 0; i < arMerged.length; i++)
    {
    	 arMerged[i].nStartPage =  arMerged[i].nStartPage - nStartPage + 1
    	arMerged[i].nEndPage = arMerged[i].nEndPage - nStartPage + 1
    }
}

function RemoveLinks(doc)
{
	for (var page = 0; page < doc.numPages; page++)
	{
		var box = doc.getPageBox("Crop", page);
		doc.removeLinks(page, box);
	}
}

app.addMenuItem({ cName: "-",cParent: "Edit", cExec: ""});
app.addMenuItem({ cName: "Transcript Marker",cParent: "Edit", cExec: "trustedProcess()"});


